from ..validators.te_validator import TEValidator
from .base_field import BaseDigits

__all__ = ["TE"]


class TE(BaseDigits):
    """
    Only Accepts string of TE with digits.

    Attributes:
        number (str): TE number.
    """

    format = "te"
    Validator = TEValidator



from typing import Any, Callable, Dict, Generator

from ..field_errors import FieldTypes, raise_field
from ..validators.base_validator import FieldMaskValidator, FieldValidator
from .base_field_v2 import BaseDigitsV2, BaseMaskV2, BasePydanticV2

__all__ = [
    "Base",
    "BaseMask",
    "BaseDigits",
]

AnyCallable = Callable[..., Any]
CallableGenerator = Generator[AnyCallable, None, None]


class Base(BasePydanticV2):
    format: str
    Validator: Callable[..., FieldValidator]

    __slots__ = ["number"]

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(type="string", format=cls.format)

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate

    @classmethod
    def validate_type(cls, value: Any) -> str:
        if not isinstance(value, str):
            raise_field(FieldTypes.type)
        return value

    @classmethod
    def validate(cls, value: str) -> str:
        doc = cls.Validator(value)
        if not doc.validate():
            raise_field(FieldTypes.invalid)
        return value


class BaseMask(Base, BaseMaskV2):
    Validator: Callable[..., FieldMaskValidator]

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_mask
        yield cls.validate

    @classmethod
    def validate_mask(cls, value: str) -> str:
        doc = cls.Validator(value)
        if not doc.validate_mask():
            raise_field(FieldTypes.mask)
        return value


class BaseDigits(Base, BaseDigitsV2):
    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_numbers
        yield cls.validate

    @classmethod
    def validate_numbers(cls, value: str) -> str:
        if not value.isdigit():
            raise_field(FieldTypes.digit)
        return value



from typing import TYPE_CHECKING

from .field_errors import *

__version__ = "1.0.1"

if TYPE_CHECKING:
    CPF = str
    CNH = str
    CPFMask = str
    CPFDigits = str
    CNPJ = str
    CNPJDigits = str
    CNPJMask = str
    TE = str
    PIS = str
    PISMask = str
    PISDigits = str
    Certidao = str
    CertidaoMask = str
    CertidaoDigits = str
else:
    from .fields.certidao_field import *
    from .fields.cnh_field import *
    from .fields.cnpj_field import *
    from .fields.cpf_field import *
    from .fields.pis_field import *
    from .fields.te_field import *



from ..validators.certidao_validator import CertidaoValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "Certidao",
    "CertidaoMask",
    "CertidaoDigits",
]


class Certidao(Base):
    """
    Accepts string of Certidao with or without mask.

    Attributes:
        number (str): Certidao number.

    """

    format = "certidao"
    Validator = CertidaoValidator


class CertidaoMask(BaseMask):
    """
    Only Accepts string of Certidao with mask.

    Attributes:
        number (str): Certidao number.
    """

    format = "certidao"
    Validator = CertidaoValidator


class CertidaoDigits(BaseDigits):
    """
    Only Accepts string of Certidao with digits.

    Attributes:
        number (str): Certidao number.
    """

    format = "certidao"
    Validator = CertidaoValidator



from ..validators.pis_validator import PISValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "PIS",
    "PISMask",
    "PISDigits",
]


class PIS(Base):
    """
    Accepts string of PIS with or without mask.

    Attributes:
        number (str): PIS number.

    """

    format = "pis"
    Validator = PISValidator


class PISMask(BaseMask):
    """
    Only Accepts string of PIS with mask.

    Attributes:
        number (str): PIS number.
    """

    format = "pis mask"
    Validator = PISValidator


class PISDigits(BaseDigits):
    """
    Only Accepts string of PIS with digits.

    Attributes:
        number (str): PIS number.
    """

    format = "pis digits"
    Validator = PISValidator



from abc import ABC, abstractmethod


class FieldValidator(ABC):
    @abstractmethod
    def validate(self):
        ...


class FieldMaskValidator(FieldValidator):
    @abstractmethod
    def validate_mask(self):
        ...



import re
from typing import List

from .base_validator import FieldValidator

__all__ = ["TEValidator"]


class TEValidator(FieldValidator):
    def __init__(self, te: str) -> None:
        self.te = te
        self.first_check_digit_weights = list(range(2, 10))
        self.second_check_digit_weights = list(range(7, 10))
        self.first_check_digit_doc_slice = slice(0, 8)
        self.second_check_digit_doc_slice = slice(8, 10)

    def validate(self) -> bool:
        te_numbers = list(re.sub("[^0-9]", "", str(self.te)))
        te = [int(n) for n in te_numbers]
        if len(set(te)) == 1:
            return False

        if len(te) != 12:
            return False

        first_check_digit = self._validate_first_digit(te)
        second_check_digit = self._validate_second_digit(
            te=te, first_check_digit=first_check_digit
        )
        return first_check_digit == te[-2] and second_check_digit == te[-1]

    def _validate_first_digit(self, te: List[int]) -> int:
        doc_digits_to_consider = te[self.first_check_digit_doc_slice]
        terms = [
            int(doc_digit * multiplier)
            for doc_digit, multiplier in zip(
                doc_digits_to_consider, self.first_check_digit_weights
            )
        ]

        total = sum(terms)

        if total % 11 == 10:
            return 0

        return total % 11

    def _validate_second_digit(self, te: List[int], first_check_digit: int) -> int:
        doc_digits_to_consider = te[self.second_check_digit_doc_slice] + [
            first_check_digit
        ]
        terms = [
            int(doc_digit * multiplier)
            for doc_digit, multiplier in zip(
                doc_digits_to_consider, self.second_check_digit_weights
            )
        ]

        total = sum(terms)

        if total % 11 == 10:
            return 0

        return total % 11



from ..validators.cpf_validator import CPFValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "CPF",
    "CPFMask",
    "CPFDigits",
]


class CPF(Base):
    """
    Accepts string of CPF with or without mask.

    Attributes:
        number (str): CPF number.

    """

    format = "cpf"
    Validator = CPFValidator


class CPFMask(BaseMask):
    """
    Only Accepts string of CPF with mask.

    Attributes:
        number (str): CPF number.
    """

    format = "cpf mask"
    Validator = CPFValidator


class CPFDigits(BaseDigits):
    """
    Only Accepts string of CPF with digits.

    Attributes:
        number (str): CPF number.
    """

    format = "cpf digits"
    Validator = CPFValidator



import re

from .base_validator import FieldMaskValidator

__all__ = ["CertidaoValidator"]


class CertidaoValidator(FieldMaskValidator):
    def __init__(self, certidao) -> None:
        self.certidao = certidao

    def validate_mask(self) -> bool:
        if len(self.certidao) == 40:
            if (
                self.certidao[6:7] == "."
                and self.certidao[9:10] == "."
                and self.certidao[12:13] == "."
                and self.certidao[17:18] == "."
                and self.certidao[19:20] == "."
                and self.certidao[25:26] == "."
                and self.certidao[29:30] == "."
                and self.certidao[37:38] == "-"
            ):
                return True
        return False

    def validate(self) -> bool:
        certidao = re.sub("[^0-9]", "", self.certidao)

        if len(set(certidao)) == 1:
            return False

        if len(certidao) != 32:
            return False

        num = list(certidao[:-2])
        dv = certidao[-2:]

        expected_dv = self._verifying_digit(num)

        if dv == expected_dv:
            return True

        return False

    def _verifying_digit(self, doc: list) -> str:
        dv1 = self._weighted_sum(doc) % 11
        dv1 = 1 if dv1 > 9 else dv1

        dv2 = self._weighted_sum(doc + [dv1]) % 11
        dv2 = 1 if dv2 > 9 else dv2

        return str(dv1) + str(dv2)

    def _weighted_sum(self, value) -> int:
        sum = 0
        multiplier = 32 - len(value)

        for i in range(len(value)):
            sum += int(value[i]) * multiplier

            multiplier += 1
            multiplier = 0 if multiplier > 10 else multiplier

        return sum



from enum import Enum

from .get_versions import get_pydantic_version

__all__ = [
    "FieldTypeError",
    "FieldMaskError",
    "FieldDigitError",
    "FieldInvalidError",
    "raise_field",
]


pydantic_version = get_pydantic_version()


class FieldTypes(Enum):
    type = "type"
    mask = "mask"
    digit = "digit"
    invalid = "invalid"


class FieldTypeError:
    code = "not_str"
    msg_template = "Input should be a valid string"
    message_template = msg_template


class FieldMaskError:
    code = "invalid_mask"
    msg_template = "invalid mask format"
    message_template = msg_template


class FieldDigitError:
    code = "not_digits"
    msg_template = "field only accept digits as string"
    message_template = msg_template


class FieldInvalidError:
    code = "invalid_data"
    msg_template = "invalid data"
    message_template = msg_template


def raise_error(code: str, msg_template: str):  # type: ignore
    if pydantic_version.value == 1:
        from pydantic import PydanticTypeError

        PydanticTypeError.code = code
        PydanticTypeError.msg_template = msg_template
        PydanticTypeError.message_template = msg_template
        raise PydanticTypeError()

    if pydantic_version.value == 2:
        from pydantic_core import PydanticCustomError

        PydanticCustomError.code = code
        PydanticCustomError.msg_template = msg_template
        PydanticCustomError.message_template = msg_template
        raise PydanticCustomError(code, msg_template)


def raise_field(context: FieldTypes):
    field_types = {
        FieldTypes.type: (FieldTypeError.code, FieldTypeError.msg_template),
        FieldTypes.mask: (FieldMaskError.code, FieldMaskError.msg_template),
        FieldTypes.digit: (FieldDigitError.code, FieldDigitError.msg_template),
        FieldTypes.invalid: (FieldInvalidError.code, FieldInvalidError.msg_template),
    }

    raise_error(*field_types.get(context))



from ..validators.cnh_validator import CNHValidator
from .base_field import BaseDigits

__all__ = ["CNH"]


class CNH(BaseDigits):
    """
    Only Accepts string of CNH with digits.

    Attributes:
        number (str): CNH number.
    """

    format = "cnh"
    Validator = CNHValidator



import re

from .base_validator import FieldValidator

__all__ = ["CNHValidator"]


class CNHValidator(FieldValidator):
    def __init__(self, cnh: str) -> None:
        self.cnh = cnh

    def validate(self) -> bool:
        cnh = re.sub("[^0-9]", "", str(self.cnh))

        if len(set(cnh)) == 1:
            return False

        if len(cnh) != 11:
            return False

        first_digit = self._validate_first_digit(cnh)
        second_digit = self._validate_second_digit(cnh)
        return cnh[9] == first_digit and cnh[10] == second_digit

    def _validate_first_digit(self, cnh: str) -> str:
        self.dsc = 0
        sum = 0

        for i in range(9, 0, -1):
            sum += int(cnh[9 - i]) * i

        first_digit = sum % 11
        if first_digit >= 10:
            first_digit, self.dsc = 0, 2
        return str(first_digit)

    def _validate_second_digit(self, cnh: str) -> str:
        sum = 0

        for i in range(1, 10):
            sum += int(cnh[i - 1]) * i

        rest = sum % 11

        second_digit = rest - self.dsc
        if second_digit < 0:
            second_digit += 11
        if second_digit >= 10:
            second_digit = 0
        return str(second_digit)



from ..validators.cnpj_validator import CNPJValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "CNPJ",
    "CNPJMask",
    "CNPJDigits",
]


class CNPJ(Base):
    """
    Accepts string of CNPJ with or without mask.

    Attributes:
        number (str): CNPJ number.

    """

    format = "cnpj"
    Validator = CNPJValidator


class CNPJMask(BaseMask):
    """
    Only Accepts string of CNPJ with mask.

    Attributes:
        number (str): CNPJ number.
    """

    format = "cnpj"
    Validator = CNPJValidator


class CNPJDigits(BaseDigits):
    """
    Only Accepts string of CNPJ with digits.

    Attributes:
        number (str): CNPJ number.
    """

    format = "cnpj"
    Validator = CNPJValidator



import re
from typing import List

from .base_validator import FieldMaskValidator

__all__ = ["PISValidator"]


class PISValidator(FieldMaskValidator):
    def __init__(self, pis: str) -> None:
        self.pis = pis

    def validate_mask(self) -> bool:
        if len(self.pis) == 14:
            if (
                self.pis[3:4] == "."
                and self.pis[9:10] == "."
                and self.pis[12:13] == "-"
            ):
                return True
        return False

    def validate(self) -> bool:
        pis_numbers = list(re.sub("[^0-9]", "", str(self.pis)))
        pis = [int(n) for n in pis_numbers]

        if len(set(pis)) == 1:
            return False

        if len(pis) != 11:
            return False

        return self._validate_digit(pis)

    def _validate_digit(self, pis: List) -> bool:
        multipliers = [3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        summation = 0

        for position in range(0, 10):
            summation += int(pis[position]) * multipliers[position]

        module = summation % 11
        dv = 11 - module

        if dv in [10, 11]:
            dv = 0

        return dv == pis[10]



from __future__ import annotations

from typing import Any, Dict, Mapping

from ..get_versions import get_pydantic_version

pydantic_version = get_pydantic_version()


if pydantic_version.value == 2:
    try:
        from pydantic_core import core_schema  # type: ignore
    except ModuleNotFoundError:
        raise ModuleNotFoundError("Are you sure you installed pydantic_core")

CoreSchema: Any = Mapping[str, Any]
JsonSchemaValue = Dict[str, Any]


class BasePydanticV2:
    format = "generic"

    @classmethod
    def validate_type(cls, value: Any):
        ...

    @classmethod
    def validate(cls, value: str):
        ...

    @classmethod
    def validate_mask(cls, value: str):
        ...

    @classmethod
    def validate_numbers(cls, value: str):
        ...

    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        source,
        *args,
        **kwargs,
    ) -> CoreSchema:
        try:
            schema = core_schema.with_info_after_validator_function(
                cls._validate, core_schema.str_schema()
            )
        except Exception:
            schema = core_schema.general_after_validator_function(
                cls._validate, core_schema.str_schema()
            )
        return schema

    @classmethod
    def __get_pydantic_json_schema__(
        cls,
        core_schema: CoreSchema,
        handler,
        *args,
        **kwargs,
    ) -> JsonSchemaValue:
        field_schema = handler(core_schema)
        field_schema.update(type="string", format=cls.format)
        return field_schema

    @classmethod
    def _validate(cls, __input_value: str, *args, **kwargs) -> str:
        cls.validate_type(__input_value)
        return cls.validate(__input_value)


class BaseMaskV2(BasePydanticV2):
    @classmethod
    def _validate(cls, __input_value: str, *args, **kwargs) -> str:
        cls.validate_type(__input_value)
        cls.validate_mask(__input_value)
        return cls.validate(__input_value)


class BaseDigitsV2(BasePydanticV2):
    @classmethod
    def _validate(cls, __input_value: str, *args, **kwargs) -> str:
        cls.validate_type(__input_value)
        cls.validate_numbers(__input_value)
        return cls.validate(__input_value)
